==> 1.0.0.tar.gz: bashy2/bits/vitalbits.sh <==
# Vital bits of bashy2

# This file is part of bashy2, an irc bot used on irc.sdf.org.
# Copyright (C) 2015 mlaine@sdfeu.org, grugly@sdf.org

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.

# You should have received a copy of the GNU General Public License along
# with this program. If not, see http://www.gnu.org/licenses/.

#---------------------------------------------------------------------------

function parse {
    local x="${1%\!*}"
    local y="${1%@*}"
    nick="${x:1}"
    user="${y#*\~}"
    host="${1#*.}"
    chan="$3"
    req="${4:1}"
    cmd="${req:1}"
    args="${*:5}"
    prefix="${req:0:1}"
}

function multiply {
    local x="$(printf "%"$1"s")"
    local y="${x// /"$2" }"
    printf '%s\n' "${y% *}"
}

function join {
    if (( "$#" == 1 )); then
        send 'join' "$1"
        chmode "$1"
    else
        quiet=1
        local chans=('./bits/channels/'*)
        local chlist="${chans[*]##*/}"
        send 'join' "${chlist// /,}"
    fi
}

function chmode {
    if (( "$#" == 1 )); then
        local chans="$1"
    else
        local chans=('./bits/channels/'*)
    fi
    if [[ "$1" = 'list' ]]; then
        send 'pm' "$2" "Channels: ${chans[*]##*/}"
    else
        for channel in "${chans[@]##*/}"; do
            local mode='+nt-ikLOpQRsVz'
            if [[ "$channel" = '#helpdesk' ]]; then
                local mode="${mode/+/+S}"
            fi
            send 'mode' "$channel" "$mode"
        done
    fi
}

function topic {
    if (( "$#" == 1 )); then
        local chans="$1"
    elif ! (( "$#" )); then
        local chans=('./bits/channels/'*)
    fi
    if (( "$#" == 2 )); then
        send 'topic' "$1" "$2"
    else
        for channel in "${chans[@]##*/}"; do
            local topic="$(cat "./bits/channels/$channel" 2>/dev/null)"
            if [[ -n "$topic" ]]; then
                send 'topic' "$channel" "$topic"
            fi
        done
    fi
}

function log {
    printf '%s %s\n' "$(date +%x\ %T)" "$*" >> "$logfile"
}

function send {
    log "${@:2}"
    case "$1" in
        'pong')
            printf 'PONG %s\n' "$2" >&3 ;;
        'pm')
            printf 'PRIVMSG %s :%s\n' "$2" "$3" >&3 ;;
        'mode')
            printf 'MODE %s %s %s\n' "$2" "$3" "$4" >&3 ;;
        'join')
            printf 'JOIN %s\n' "$2" >&3 ;;
        'part')
            printf 'PART %s\n' "$2" >&3 ;;
        'oper')
            printf 'OPER %s %s\n' "$2" "$3" >&3 ;;
        'user')
            printf 'USER %s 0 * :%s\n' "$2" "$3" >&3 ;;
        'nick')
            printf 'NICK %s\n' "$2" >&3 ;;
        'topic')
            printf 'TOPIC %s :%s\n' "$2" "$3" >&3 ;;
    esac
}

function auth {
    if [[ "$2" = "${auth_host}" ]]; then
        if [[ "$user" = "$bot_user" ]]; then
            eval "$1"='aohv'
        elif $(id "$3" | grep -qw '\,600(MetaARPA)'); then
            eval "$1"='ohv'
        elif $(id "$3" | grep -qw '500(arpa)'); then
            eval "$1"='hv'
        else
            eval "$1"='v'
        fi
    elif [[ "$2" = "${auth_host2}" ]]; then
        eval "$1"='v'
    else
        eval "$1"=''
    fi
}

function serv {
    case "$2" in
        'list')
            chmode 'list' "$1" ;;
        'add')
            local ch="${3%% *}"
            join "$ch"
            if [[ "$3" =~ ' ' ]]; then
                local topic="${3#* }"
                topic "$ch" "$topic"
            fi
            printf '%s\n' "$topic" > "./bits/channels/$ch"
            send 'pm' "$1" "Added $ch" ;;
        'del')
            local ch="${3%% *}"
            rm "./bits/channels/$ch"
            send 'part' "$ch"
            send 'pm' "$1" "Deleted $ch" ;;
        'chmode')
            chmode
            send 'pm' "$1" 'Channel modes set.' ;;
        'chtopic')
            topic
            send 'pm' "$1" 'Channel topics set.' ;;
        'reload')
            source './bits/vitalbits.sh'
            send 'pm' "$1" 'Reloaded!' ;;
        'quiet')
            quiet=0
            send 'pm' "$1" 'I will shut up.' ;;
        'loud')
            quiet=1
            send 'pm' "$1" 'Volume up!' ;;
        'die')
            send 'pm' "$1" "What'd I do?"
            exit 0 ;;
    esac
}

function run {
    if [[ -x "$2/$3" ]]; then
        ( cd "$2" && "./$3" "$1" "$4" )
    else
        send 'pm' "$1" "'$3' not found. See !commands and ?triggers."
    fi
}

function priv {
    parse "$@"
    if [[ "$req" = '!up' ]]; then
        auth 'flag' "$host" "$user"
        if [[ -n "$flag" ]]; then
            local name="$(multiply "${#flag}" "$nick")"
            local chans=('./bits/channels/'*)
            for channel in "${chans[@]##*/}"; do
                send 'mode' "$channel" "+$flag" "$name"
            done
        fi
        return 0
    elif [[ "$chan" = "$bot_user" ]]; then
        if [[ "$user" = "$bot_user2" ]]; then
            quiet=0
            return 0
        fi
        chan="$user"
    fi
    if [[ -n "$cmd" && ("$quiet" -eq 1 || "$chan" = "$user") ]]; then
        case "$prefix" in
            '!')
                run "$chan" "$cmd_dir" "$cmd" "$args"
                return 0 ;;
            '?')
                run "$chan" "$trig_dir" "$cmd" "$args"
                return 0 ;;
        esac
    fi
    if [[ "$prefix" = '%' && -f "./bits/maintainers/$user" ]]; then
        auth 'flag' "$host" "$user"
        [[ -n "$flag" ]] && serv "$chan" "$cmd" "$args"
    fi
}

function user_join {
    parse "$@"
    auth 'flag' "$host" "$user"
    if [[ -n "$flag" ]]; then
        local ch="${chan:1}" 
        local name="$(multiply "${#flag}" "$nick")"
        send 'mode' "$ch" "+$flag" "$name"
        if [[ "$user" = "$bot_user" ]]; then
            chmode "$ch"
            topic "$ch"
        fi
    fi
}

function user_quit {
    parse "$@"
    if [[ "$user" = "$bot_user2" ]]; then
        quiet=1
    fi
}

==> 1.1.0.tar.gz: bashy2/bits/vitalbits.sh <==
# Vital bits of bashy2

# This file is part of bashy2, an irc bot used on irc.sdf.org.

# Copyright (C) 2015-2016 mlaine@sdfeu.org, grugly@sdf.org

# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.

# You should have received a copy of the GNU General Public License along with
# this program. If not, see http://www.gnu.org/licenses/.

#------------------------------------------------------------------------------

function parse {
    local x="${1%\!*}"
    local y="${1%@*}"
    nick="${x:1}"
    user="${y#*\~}"
    host="${1#*.}"
    chan="$3"
    req="${4:1}"
    cmd="${req:1}"
    args="${*:5}"
    prefix="${req:0:1}"
}

function multiply {
    local x="$(printf "%"$1"s")"
    local y="${x// /"$2" }"
    printf '%s\n' "${y% *}"
}

function join {
    if (( "$#" == 1 )); then
        send 'join' "$1"
        chmode "$1"
    else
        quiet=1
        local chans=('./bits/channels/'*)
        local chlist="${chans[*]##*/}"
        send 'join' "${chlist// /,}"
    fi
}

function chmode {
    if (( "$#" == 1 )); then
        local chans="$1"
    else
        local chans=('./bits/channels/'*)
    fi
    if [[ "$1" = 'list' ]]; then
        send 'pm' "$2" "Channels: ${chans[*]##*/}"
    else
        for channel in "${chans[@]##*/}"; do
            local mode='+nt-ikLOpQRsVz'
            if [[ "$channel" = '#helpdesk' ]]; then
                local mode="${mode/+/+S}"
            fi
            send 'mode' "$channel" "$mode"
        done
    fi
}

function topic {
    if (( "$#" == 1 )); then
        local chans="$1"
    elif (( ! "$#" )); then
        local chans=('./bits/channels/'*)
    fi
    if (( "$#" == 2 )); then
        send 'topic' "$1" "$2"
    else
        for channel in "${chans[@]##*/}"; do
            local topic="$(cat "./bits/channels/$channel" 2>/dev/null)"
            if [[ -n "$topic" ]]; then
                send 'topic' "$channel" "$topic"
            fi
        done
    fi
}

function log {
    printf '%s %s\n' "$(date +%x\ %T)" "$*" >> "$logfile"
}

function send {
    log "${@:2}"
    case "$1" in
        'pong')
            printf 'PONG %s\n' "$2" >&3 ;;
        'pm')
            printf 'PRIVMSG %s :%s\n' "$2" "$3" >&3 ;;
        'mode')
            printf 'MODE %s %s %s\n' "$2" "$3" "$4" >&3 ;;
        'join')
            printf 'JOIN %s\n' "$2" >&3 ;;
        'part')
            printf 'PART %s\n' "$2" >&3 ;;
        'oper')
            printf 'OPER %s %s\n' "$2" "$3" >&3 ;;
        'user')
            printf 'USER %s 0 * :%s\n' "$2" "$3" >&3 ;;
        'nick')
            printf 'NICK %s\n' "$2" >&3 ;;
        'topic')
            printf 'TOPIC %s :%s\n' "$2" "$3" >&3 ;;
    esac
}

function auth {
    if [[ "$2" = "${auth_host}" ]]; then
        if [[ "$user" = "$bot_user" ]]; then
            eval "$1"='aohv'
        elif $(id "$3" | grep -qw '\,600(MetaARPA)'); then
            eval "$1"='ohv'
        elif $(id "$3" | grep -qw '500(arpa)'); then
            eval "$1"='hv'
        else
            eval "$1"='v'
        fi
    elif [[ "$2" = "${auth_host2}" ]]; then
        eval "$1"='v'
    else
        eval "$1"=''
    fi
}

function serv {
    case "$2" in
        'list')
            chmode 'list' "$1" ;;
        'add')
            local ch="${3%% *}"
            join "$ch"
            if [[ "$3" =~ ' ' ]]; then
                local topic="${3#* }"
                topic "$ch" "$topic"
            fi
            printf '%s\n' "$topic" > "./bits/channels/$ch"
            send 'pm' "$1" "Added $ch" ;;
        'del')
            local ch="${3%% *}"
            rm "./bits/channels/$ch"
            send 'part' "$ch"
            send 'pm' "$1" "Deleted $ch" ;;
        'chmode')
            chmode
            send 'pm' "$1" 'Channel modes set.' ;;
        'chtopic')
            topic
            send 'pm' "$1" 'Channel topics set.' ;;
        'reload')
            source './bits/vitalbits.sh'
            send 'pm' "$1" 'Reloaded!' ;;
        'quiet')
            quiet=0
            send 'pm' "$1" 'I will shut up.' ;;
        'loud')
            quiet=1
            send 'pm' "$1" 'Volume up!' ;;
        'die')
            send 'pm' "$1" "What'd I do?"
            exit 0 ;;
    esac
}

function run {
    if [[ -x "$2/$3" ]]; then
        ( cd "$2" && "./$3" "$1" "$4" )
    else
        send 'pm' "$1" "'$3' not found. See !commands and ?triggers."
    fi
}

function priv {
    parse "$@"
    if [[ "$req" = '!up' ]]; then
        auth 'flag' "$host" "$user"
        if [[ -n "$flag" ]]; then
            local name="$(multiply "${#flag}" "$nick")"
            local chans=('./bits/channels/'*)
            for channel in "${chans[@]##*/}"; do
                send 'mode' "$channel" "+$flag" "$name"
            done
        fi
        return 0
    elif [[ "$chan" = "$bot_user" ]]; then
        if [[ "$user" = "$bot_user2" ]]; then
            quiet=0
            return 0
        fi
        chan="$user"
    fi
    if [[ -n "$cmd" && ! "$cmd" =~ [^[:alnum:]_]\
            && ("$quiet" -eq 1 || "$chan" = "$user") ]]; then
        case "$prefix" in
            '!')
                run "$chan" "$cmd_dir" "$cmd" "$args"
                return 0 ;;
            '?')
                run "$chan" "$trig_dir" "$cmd" "$args"
                return 0 ;;
        esac
    fi
    if [[ "$prefix" = '%' && -f "./bits/maintainers/$user" ]]; then
        auth 'flag' "$host" "$user"
        [[ -n "$flag" ]] && serv "$chan" "$cmd" "$args"
    fi
}

function user_join {
    parse "$@"
    auth 'flag' "$host" "$user"
    if [[ -n "$flag" ]]; then
        local ch="${chan:1}" 
        local name="$(multiply "${#flag}" "$nick")"
        send 'mode' "$ch" "+$flag" "$name"
        if [[ "$user" = "$bot_user" ]]; then
            chmode "$ch"
            topic "$ch"
        fi
    fi
}

function user_quit {
    parse "$@"
    if [[ "$user" = "$bot_user2" ]]; then
        quiet=1
    fi
}
